-- VARIABLES GLOBAL TO ADDON

local _, L = ...						-- For localization. This is currently used only for pet type names
										-- which is necessary to make the sound system work correctly.

local nbNameFormatted = "|cFFCCCCFFNoseBag|r"
local nbGray = "|cFF777777"
local nbRed = "|cFFFF0000"
local nbYellow = "|cFFFFFF00"
local nbGreen = "|cFF00FF00"
local nbWhite = "|cFFFFFFFF"
local nbLightGray = "|cFFAAAAAA"
local nbOrange = "|cFFFF7700"
local nbLoaded = false					-- Set at the end of initialization. Prevents multiple inits.
local nbInCombat = false				-- Set when player enters combat. Reset when combat ends.
local nbPlayerDead = false				-- Set when player dies. Cleared when player resurrects.
local nbOverButton = false				-- True while mouse is over the button.
local nbShiftDown = false				-- True when shift key is down.
local nbControlDown = false				-- True when control key is down.
local nbAltDown = false					-- True when alt key is down.
local nbMinAlpha = .25					-- Alpha when button is faded out.
local nbMaxAlpha = 1.0					-- Alpha when button is faded in.
local nbFadeTime = .25					-- Length of fade in seconds.

CreateFrame("GameTooltip", "nbGameTooltip", nil, "GameTooltipTemplate")	-- Custom private tooltip so we don't
																		-- fight with other addons or the system.

	-- PLAYER_LOGIN
	
local nbPlayerLevel = 1					-- Tracks player level. Used to avoid unnecessary calls to UnitLevel.

	-- Pet
	
local nbAlertTicksRemaining = 0			-- Remaining calls to nbCheckPetHappiness before player should be alerted.
local nbAlertDelayTicksUnhappy = 4		-- Alert delay when pet happiness is 1
local nbAlertDelayTicksContent = 8		-- Alert delay when pet happiness is 2
local nbAlertDelayTicks = 8				-- Number of calls to nbCheckPetHappiness between alerts.
local nbHappinessCheckSeconds = 1		-- Time between calls to nbCheckPetHappiness.
local nbAlertSound = ""					-- Sound file to play when alerting.
local nbFeedEffectID = 1539				-- Spell ID of Pet Feed Effect.

	-- Button
local nbButtonIsVisible = true			-- Tracks button visibility to avoid unnecessary calls to Show()
local nbBtn = CreateFrame("Button",nil,UIParent,"SecureActionButtonTemplate")	-- Main button. Other setup is in
																				-- Initialize.
nbBtn:SetNormalFontObject(GameFontNormalHuge)
local f = nbBtn:GetNormalFontObject()
f:SetTextColor(1.0,1.0,1.0,1.0)
nbBtn:SetNormalFontObject(f)

local nbFeedSpellID = 6991				-- This is the spell ID of the Feed Pet Effect
local nbFeedSpellName = "Feed Pet"		-- This will contain the localized Feed Pet spell name for the macro
										-- It's necessary to do it this way because macros can't use CastSpell()


nbBtn:RegisterEvent("PLAYER_LOGIN")			-- Used to kick off initialization process.
nbBtn:RegisterEvent("PLAYER_REGEN_DISABLED")-- Fires at start of combat.
nbBtn:RegisterEvent("PLAYER_REGEN_ENABLED")	-- Fires at end of combat.
nbBtn:RegisterEvent("PLAYER_DEAD")			-- Fires when player dies.
nbBtn:RegisterEvent("PLAYER_ALIVE")			-- Fires whenplayer accepts rez before releasing.
nbBtn:RegisterEvent("PLAYER_UNGHOST")		-- Fires when player rezzes or finishes corpse run.

	-- Button State Data & Functions
	
local function nbShowButton()			-- Wrapper for Show to avoid changing button during combat.
	if not nbInCombat then
		nbButtonIsVisible = true
		nbBtn:Show()
	end
end

local function nbHideButton()			-- Wrapper for Hide to avoid changing button during combat.
	if not nbInCombat then
		nbButtonIsVisible = false
		nbBtn:Hide()
	end
end

	-- Note: Multiple interactions with the button during combat will be overwritten by the most recent one.
	-- For example, if the button is cleared and then a new food is dragged onto it while combat is
	-- underway, only the new food will be applied to the button. This means changes won't appear in the button
	-- until combat ends which makes the button seem to be disabled.
local nbPendingTexture = ""				-- Texture to be applied to button after combat if changed during combat.
local nbPendingMacro = ""				-- Macro to ba applied to button after combat if changed during combat.

-- If combat is underway then delay 1 second and retry. Otherwise, apply new
-- attributes to the button.
local function nbApplyPendingButtonAttributes()
	if nbInCombat then
		C_Timer.After(1, nbApplyPendingButtonAttributes)
	else
		nbBtn:SetNormalTexture(nbPendingTexture)
		nbBtn:SetAttribute("macrotext", nbPendingMacro)
	end
end

-- Set changes to button attributes to be applied as soon as possible.
local function nbSetPendingButtonChanges(texture, macro)
	nbPendingTexture = texture
	nbPendingMacro = macro
	nbApplyPendingButtonAttributes()
end

-- Update the button based on currently set food.
local function nbUpdateButtonState(forceButtonUpdate)
	local state, count
	
	if svActiveFood == nil then
		state = "EMPTY"
		nbBtn:SetText("")
	else
		count = GetItemCount(svActiveFood)
		if count < 1 then
			state = "EMPTY"
			nbBtn:SetText("")
		else
			state = "FEED"
			nbBtn:SetText(count)
		end
	end
	
	if state ~= svButtonState or forceButtonUpdate then	-- Update the button only when it's really necessary
		svButtonState = state
		if svButtonState == "FEED" then
			local foodName = GetItemInfo(svActiveFood)
			local icon = GetItemIcon(svActiveFood)
			nbSetPendingButtonChanges(icon, "/cast " .. nbFeedSpellName .. "\n/use " .. foodName .. "\n/script nbNotifyButtonClicked()\n")
		elseif svButtonState == "EMPTY" then
			-- Changes to the button are stored in case combat happens. We'll keep trying to apply them
			-- once per second until combat ends.
			nbSetPendingButtonChanges("Interface/ICONS/Ability_Hunter_BeastTraining.blp", "/script OpenAllBags()")
		end
	end
end

-- Called from the button macro when feeding pet to notify us to update the button.
function nbNotifyButtonClicked()
	nbUpdateButtonState(true)
end
	
-- SOUND

-- Plays a sound if the silent flag isn't set.
local function nbPlaySound(sound, channel)
	if svSoundMode ~= "SILENT" then
		PlaySoundFile(sound, channel)
	end
end

-- Set the alert sound to be played when pet is hungry. Note that pet type names
-- are localized and nbPetSounds uses localized indexes. This is because there appears
-- to be no ID-based way to identify pet families.
function nbSetAlertSound()
	if svSoundMode == "NORMAL" then
		nbAlertSound = nbPetSounds[svSoundMode]
	elseif svSoundMode == "CUSTOM" then
		nbAlertSound = nbPetSounds[svSoundMode]
	elseif svSoundMode == "PET TYPE" then
		_,_,_,f = GetStablePetInfo(0)
		if f then
			nbAlertSound = nbPetSounds[L[f]]
		else
			print(nbNameFormatted .. ": " .. nbRed .. "Missing or incorrect pet family name! Please report this.")
			nbAlertSound = nbPetSounds["NORMAL"]		-- This will happen if a pet family is mistranslated or
														-- loc for the client region doesn't exist.
		end
	end
end

-- TOOLTIP

-- Sets up the button tooltip based on its state.
local function nbUpdateTooltip()
	-- nbGameTooltip:ClearLines()
	nbGameTooltip:SetOwner(nbBtn, "ANCHOR_TOPRIGHT")	-- This does a ClearLines().
	
	local nomodColor = nbLightGray
	local shiftColor = nbLightGray
	local ctrlColor = nbLightGray
	local altColor = nbLightGray
	
	if IsShiftKeyDown() then
		shiftColor = nbWhite
	elseif IsControlKeyDown() then
		ctrlColor = nbWhite
	elseif IsAltKeyDown() then
		altColor = nbWhite
	else
		nomodColor = nbWhite
	end
	
	if svButtonState == "EMPTY" then
		local versionString = GetAddOnMetadata("NoseBag","Version")
		nbGameTooltip:AddLine(nbNameFormatted .. " " .. nbWhite .. versionString)
		nbGameTooltip:AddLine("Sound Mode: " .. svSoundMode)
		nbGameTooltip:AddDoubleLine(nbGray.."...|r", nbGray.."...|r")
		nbGameTooltip:AddLine(nomodColor .. "DRAG and DROP pet food here.")
	elseif svButtonState == "FEED" then
		if svActiveFood then
			local hps
			local itemName,_,_,itemLevel = GetItemInfo(svActiveFood)
			local levelDifference = UnitLevel("pet") - itemLevel
			local levelColor = nbGreen
			local hps = 35/2
			if levelDifference > 30 then
				levelColor = nbGray
				hps = 0
			elseif levelDifference > 20 then
				levelColor = nbRed
				hps = 8/2
			elseif levelDifference > 10 then
				levelColor = nbYellow
				hps = 17/2
			end
			
			local itemCount = GetItemCount(svActiveFood)
			nbGameTooltip:AddLine(levelColor .. itemName)
			nbGameTooltip:AddDoubleLine(L["Happiness per second: "] .. hps, L["Available: "] .. itemCount)
			nbGameTooltip:AddLine(L["Sound Mode: "] .. svSoundMode)
			nbGameTooltip:AddDoubleLine(nbGray .. "...", nbGray .. "...")
			nbGameTooltip:AddLine(nomodColor .. L["RIGHT-CLICK to clear"])
		end
	end
	
	if svSoundMode == "SILENT" then
		nbGameTooltip:AddLine(shiftColor .. L["SHIFT-RIGHT-CLICK for normal sound"])
	elseif svSoundMode == "NORMAL" then
		nbGameTooltip:AddLine(shiftColor .. L["SHIFT-RIGHT-CLICK for custom sound"])
	elseif svSoundMode == "CUSTOM" then
		nbGameTooltip:AddLine(shiftColor .. L["SHIFT-RIGHT-CLICK for pet type sound"])
	elseif svSoundMode == "PET TYPE" then
		nbGameTooltip:AddLine(shiftColor .. L["SHIFT-RIGHT-CLICK to turn sound off"])
	end
	
	if svButtonLock then
		nbGameTooltip:AddLine(ctrlColor .. L["CTRL-RIGHT-CLICK to unlock button"])
	else
		nbGameTooltip:AddLine(ctrlColor .. L["CTRL-RIGHT-CLICK to lock button"])
	end
	
	nbGameTooltip:AddLine(altColor .. L["ALT-RIGHT-CLICK to change button size"])
end

-- PET

-- Alert player that pet wants to be fed.
local function nbAlertFeed(delay, buttonScale)
	nbAlertDelayTicks = delay
	nbAlertTicksRemaining = nbAlertTicksRemaining + 1
	if nbAlertTicksRemaining >= nbAlertDelayTicks then
		nbBtn:LockHighlight()
		if not nbInCombat then
			nbPlaySound(nbAlertSound, "SFX")
			nbBtn:SetSize(svButtonSize*buttonScale, svButtonSize*buttonScale)
		end
		nbAlertTicksRemaining = 0
	else
		nbBtn:UnlockHighlight()
		nbBtn:SetSize(svButtonSize, svButtonSize)
	end
end

-- Check the pet for a Feed Pet Effect buff. Note that, while it
-- should be possible to pass "Feed Pet Effect" to UnitBuff, in
-- practice this appears not to be true. So, instead, we scan through
-- all buffs currently on the pet until we find it or run out of
-- buffs to check.
local function nbHasFeedEffect()
	local buffIndex = 1
	local buff,_,_,_,_,expirationTime,_,_,_,spellID = UnitBuff("pet", buffIndex)
	while buff do
		if spellID == nbFeedEffectID then
			return expirationTime - GetTime()	-- Return seconds left before effect expires.
		end
		buffIndex = buffIndex + 1
		buff,_,_,_,_,expirationTime,_,_,_,spellID = UnitBuff("pet", buffIndex)
	end
	return 0									-- No feed pet effect found.
end

local function nbCheckForAlert()
	-- Bail out if no chance of having a pet out.
	if UnitOnTaxi("player") then return end
	if InCinematic() then return end
	if IsFlying() then return end
	if IsStealthed() then return end
	if nbPlayerDead then return end
	
	-- Make sure we have a live pet present.
	if HasPetUI() then
		if not UnitIsDead("pet") then
			local happiness = GetPetHappiness()
			local feedDuration = nbHasFeedEffect()
			if feedDuration <= 0 then
				if happiness == 1 then
					nbAlertFeed(nbAlertDelayTicksUnhappy, 1.15)
				elseif happiness == 2 then
					nbAlertFeed(nbAlertDelayTicksContent, 1.05)
				end
			end
		end
		
		nbSetAlertSound()	-- Update pet alert sound in case we've switched pets.
	end
end

-- Called periodically to check on pet's happiness. Also checks player
-- level to see if button needs to be unhidden because player leveled.
local function nbCheckPetHappiness()
	-- Check to see if player is at least level 10.
	-- Note that nbPlayerLevel persists during the session so
	-- it can be used as a flag to enable normal button function
	-- once the player levels from 9 to 10 and to prevent
	-- redundant ShowButton calls.
	if nbPlayerLevel < 10 then
		nbPlayerLevel = UnitLevel("player")
		if nbPlayerLevel > 9 then
			if not nbButtonIsVisible then
				nbShowButton()
				nbCheckForAlert()
			end
		end
	-- This is where normal operation of the button happens.
	else
		nbCheckForAlert()
	end
	
end

-- INITIALIZE

-- Set up saved variables and configure the button. Also set up button event handlers.
local function nbInitialize()
	-- We need the localized name of the feed pet spell.
	nbFeedSpellName = GetSpellInfo(nbFeedSpellID)

	-- SAVED VARIABLES AND BUTTON SETUP
	
	if not svSoundMode then
		svSoundMode = "NORMAL"
	end
	nbSetAlertSound()
	
	if not svXpos then
		svXpos = 0
	end
	if not svYpos then
		svYpos = 540
	end
	if not svRelativePoint then
		svRelativePoint = "BOTTOM"
	end
	nbBtn:SetAttribute("type1", "macro")
	nbBtn:SetClampedToScreen(true)
	nbBtn:SetHighlightTexture("Interface/BUTTONS/ButtonHilight-Square.blp")
	nbBtn:SetMovable(true)
	nbBtn:EnableMouse(true)
	nbBtn:RegisterForDrag("LeftButton")
	nbBtn:SetPoint(svRelativePoint,svXpos,svYpos)
	nbBtn:SetAlpha(nbMaxAlpha)
	
	if not svButtonSize then
		svButtonSize = 64
	end
	nbBtn:SetWidth(svButtonSize)
	nbBtn:SetHeight(svButtonSize)

	if not svButtonState then
		svActiveFood = nil
		svButtonState = "EMPTY"
	end
	nbUpdateButtonState(true)
	
	-- BUTTON DRAG
	
	nbBtn:SetScript("OnDragStart", function()
		if not svButtonLock then
			nbBtn:StartMoving()
		end
	end)
	
	nbBtn:SetScript("OnDragStop", function()
		nbBtn:StopMovingOrSizing()
		_,_,svRelativePoint,svXpos,svYpos = nbBtn:GetPoint()
	end)
	
	nbBtn:SetScript("OnReceiveDrag", function()
		local infoType, itemID = GetCursorInfo()
		if infoType == "item" then
			svActiveFood = itemID
			nbUpdateButtonState(true)
			nbUpdateTooltip()
			local _,_,_,petClass = GetStablePetInfo(0)
			nbGameTooltip:Show()
		end
		ClearCursor()
	end)
	
	-- BUTTON UPDATE

	nbBtn:SetScript("OnUpdate", function()
		nbUpdateButtonState(false)
		
		if not nbInCombat then
			-- If mouse is over the button then check to see if the state of any mod key has changed state.
			-- Update the tooltip if so.
			if nbOverButton then
				local s = IsShiftKeyDown()
				local c = IsControlKeyDown()
				local a = IsAltKeyDown()
				
				if s ~= nbShiftDown
				or c ~= nbControlDown
				or a ~= nbAltDown then
					nbGameTooltip:Hide()
					nbUpdateTooltip()
					nbGameTooltip:Show()
				end
				
				nbShiftDown = s
				nbControlDown = c
				nbAltDown = a
			end
		end
	end)
	
	-- BUTTON MOUSE DOWN HANDLER
	
	nbBtn:SetScript("OnMouseDown", function(self, button)
		nbBtn:SetHighlightTexture("interface/buttons/CheckButtonHilight.blp")
	end)
	
	-- BUTTON RIGHT CLICK HANDLING

	nbBtn:SetScript("OnMouseUp", function(self, button)
		nbBtn:SetHighlightTexture("Interface/BUTTONS/ButtonHilight-Square.blp")
		if button == "RightButton" then
			if nbBtn:IsMouseOver() then
				if IsShiftKeyDown() then
					if svSoundMode == "NORMAL" then
						svSoundMode = "CUSTOM"
					elseif svSoundMode == "CUSTOM" then
						svSoundMode = "PET TYPE"
					elseif svSoundMode == "PET TYPE" then
						svSoundMode = "SILENT"
					else
						svSoundMode = "NORMAL"
					end
					nbSetAlertSound()
					nbUpdateTooltip()
					nbGameTooltip:Show()
				elseif IsControlKeyDown() then
					svButtonLock = not svButtonLock
					nbUpdateTooltip()
					nbGameTooltip:Show()
				elseif IsAltKeyDown() then
					if svButtonSize == 96 then
						svButtonSize = 64
					elseif svButtonSize == 64 then
						svButtonSize = 48
					elseif svButtonSize == 48 then
						svButtonSize = 32
					elseif svButtonSize == 32 then
						svButtonSize = 96
					else
						svButtonSize = 48
					end
					nbBtn:SetWidth(svButtonSize)
					nbBtn:SetHeight(svButtonSize)
				else
					svActiveFood = nil
					nbUpdateButtonState(true)
					nbUpdateTooltip()
					nbGameTooltip:Show()
				end
			end
			nbPlaySound("Sound/Interface/iUiInterfaceButtonA.ogg","SFX")
		end
	end)
	
	-- TOOLTIP
	
	nbBtn:SetScript("OnEnter", function(self)
		if nbInCombat then
			nbGameTooltip:Hide()
		else
			nbUpdateTooltip()
			nbGameTooltip:Show()
			nbOverButton = true
		end
	end)
	
	nbBtn:SetScript("OnLeave", function(self)
		nbGameTooltip:Hide()
		nbOverButton = false
		self:SetHighlightTexture("Interface/BUTTONS/ButtonHilight-Square.blp")
	end)
end

-- Called at startup to verify player's class and level. Hides button
-- if it's not needed.
local function nbCheckOkToRun()
	-- Check to see if this region is supported by localization and set the
	-- region code color so players can see if their region is supported.
	local regionString = GetLocale()
	local locColor = nbRed
	if L[regionString] == true then
		locColor = nbGreen
	end
	nbNameFormatted = nbNameFormatted .. locColor .. " (" .. regionString .. ")|r"
	
	-- If we're not on a hunter, don't bother. If this is a hunter below
	-- level 10 then just hide the button for now.
	local _,_,classIndex = UnitClass("player")

	if classIndex == 3 then
		local _,_,notes = GetAddOnInfo("NoseBag")
		print(nbNameFormatted .. ": " .. notes)
		
		nbInitialize()
		C_Timer.NewTicker(nbHappinessCheckSeconds, nbCheckPetHappiness)
		
		nbPlayerLevel = UnitLevel("player")
		if nbPlayerLevel > 9 then
			nbShowButton()
		else
			print(nbNameFormatted .. ": Below level 10. Hiding button.")
			nbHideButton()
		end
	else
		nbHideButton()
	end
end

-- MAIN EVENT HANDLER

nbBtn:SetScript("OnEvent", function(self, event, ...)
	if event == "PLAYER_LOGIN" then
		if not nbLoaded then
			C_Timer.After(1, nbCheckOkToRun)
			nbLoaded = true
		end
	elseif event == "PLAYER_REGEN_DISABLED" then
		UIFrameFadeIn(self, nbFadeTime, nbMaxAlpha, nbMinAlpha)
		nbInCombat = true
	elseif event == "PLAYER_REGEN_ENABLED" then
		UIFrameFadeIn(self, nbFadeTime, nbMinAlpha, nbMaxAlpha)
		nbInCombat = false
	elseif event == "PLAYER_DEAD" then
		nbPlayerDead = true
	elseif event == "PLAYER_ALIVE" or event == "PLAYER_UNGHOST" then
		nbPlayerDead = false
	end
end)
